/*********************************************************
* function.c - functions for meansuring
*
* v_0.1
*
* Copyright (c)2014 Matej Z. All rights reserved.
* matej.zdenek@gmail.com
*********************************************************/

#include "dynamZero.h"
#include <math.h>

void DynamZeroClearStatistics(tStatDynamZero *stats, int onlyActual)
{
	for (int i = 0; i < MATRIX_SIZE; i++)
	{
		stats->histogramActual[i] = 0;

		if (!onlyActual)
		{
			stats->histogramTotal[i] = 0;
		}
	}
}


int DynamZeroPrintStatistics(char* s, int printHeader, unsigned int* histogram, char* text)
{
	int charNum = 0;

	long long sum = 0;
	long long pow2sum = 0;
	long long count = 0;

	for (int i = 0; i < MATRIX_SIZE; i++)
	{
		sum = sum + (long long)histogram[i]*(long long)i;
		pow2sum = pow2sum + (long long)histogram[i] * (long long)i * (long long)i;
		count = count + (long long)histogram[i];
	}

	//header
	if (printHeader)
	{
		charNum += sprintf(s + charNum, "dynam nula  |9SIG|     |3SIG|     |MEAN|     |3SIG|     |9SIG|           \n");
		charNum += sprintf(s + charNum, "-------------------------------------------------------------------------\n");
	}

	if (sum == 0 || count == 0)
	{
		charNum += sprintf(s + charNum, "no data to print statistics\n");
		return charNum;
	}

	double mean = sum / (double)count;
	double std = sqrt( (pow2sum/(double)count) - mean*mean );

	//make easy histogram to print in console
	//make 6 areas defined using mean and 1 and 3 sigma

	//define borders using mean and std
	int borders[6];
	borders[0] = mean - 9*std; 	//minus 3 sigma
	borders[0] = (borders[0] < 0) ? 0 : borders[0];

	borders[1] = mean - 3*std;	//minus 1 sigma
	borders[1] = (borders[1] < 0) ? 0 : borders[1];

	borders[2] = mean;			//mean

	borders[3] = mean + 3*std;	//plus 1 sigma
	borders[3] = (borders[3] >= MATRIX_SIZE) ? MATRIX_SIZE-1 : borders[3];

	borders[4] = mean + 9*std;	//plus 3 sigma
	borders[4] = (borders[4] >= MATRIX_SIZE) ? MATRIX_SIZE-1 : borders[4];

	borders[5] = MATRIX_SIZE-1;	//stop

	//make sum in areas
	int countInArea[6];
	for (int i = 0; i < 6; i++)
	{
		countInArea[i] = 0;
	}
	int area = 0;
	for (int i = 0; i < MATRIX_SIZE; i++)
	{
		if (i > borders[area]) area++;

		countInArea[area] += histogram[i];
	}

	//compute percents in areas
	float percentInAreas[6];
	for (int i = 0; i < 6; i++)
	{
		percentInAreas[i] = countInArea[i] / (float)count * 100;
	}

	//print results
	charNum += sprintf(s + charNum, "    |0|");
	for (int i = 0; i < 6; i++)
	{
		charNum += sprintf(s + charNum, "     |%4d|", borders[i]);
	}
	charNum += sprintf(s + charNum, "\n%3s   ", text);
	for (int i = 0; i < 6; i++)
	{
		charNum += sprintf(s + charNum, "|%4.1f%%|    ", percentInAreas[i]);
	}
	charNum += sprintf(s + charNum, "\n-------------------------------------------------------------------------\n");

	return charNum;
}




unsigned char DynamZero(float *impuls, int delka, tStatDynamZero *stats)
{
	int a;

	int prvniVzorek = 5;
	int pocetVzorkuNaNulu = 15;


	float nula = 0;
	//min = 99999;
	for (a = prvniVzorek; a < prvniVzorek+pocetVzorkuNaNulu; a++)
	{
		nula += impuls[a];
	}
	//for (a=0;a<delka;a++){
	//	if (min>impuls[a]) min = impuls[a];
	//}
	nula = nula / (float)pocetVzorkuNaNulu;

	for (a=0;a<delka;a++){
		impuls[a] = impuls[a] - nula;
		//if (impuls[a]<0) impuls[a]=0;
		//impuls[a] = impuls[a] - min;


	}

	//printf("Posun nuly o %f\r\n",nula);


	//statistika
	int nulaStats = (int)nula;
	nulaStats = (nulaStats < 0) ? 0 : ((nulaStats >= MATRIX_SIZE) ? (MATRIX_SIZE-1) : nulaStats);
	stats->histogramActual[nulaStats]++;
	stats->histogramTotal[nulaStats]++;

	return(0);
}
