/*********************************************************
* function.c - functions for meansuring
*
* v_0.1
*
* Copyright (c)2014 Matej Z. All rights reserved.
* matej.zdenek@gmail.com
*********************************************************/

#include "energy.h"
#include "constants.h"
#include <stdio.h>
#include <math.h>



float MaximumImpulsu(float *impuls, int *max, int delka)
{
	int a, tmp;
	float maximum = 0;

	tmp = 0;
	for (a=0; a<delka; a++)
	{
		if (impuls[a] > maximum)
		{
			maximum = impuls[a];
			tmp = a;
		}
	}

	*max = tmp;
	return(maximum);
}

void EnergyClearStatistics(tStatEnergy *stats, int onlyActual)
{
	if (!onlyActual)
	{
		stats->total.minPulseLength = 9999999;
		stats->total.sumPulseLength = 0;
		stats->total.sumPulseLengthPow2 = 0;
		stats->total.countPulses = 0;
		stats->total.numToShortPulsesAtBegining = 0;
		stats->total.numToShortPulsesAtEnding = 0;
		stats->total.numSaturatedIntegrals = 0;
	}

	stats->actual.minPulseLength = 9999999;
	stats->actual.sumPulseLength = 0;
	stats->actual.sumPulseLengthPow2 = 0;
	stats->actual.countPulses = 0;
	stats->actual.numToShortPulsesAtBegining = 0;
	stats->actual.numToShortPulsesAtEnding = 0;
	stats->actual.numSaturatedIntegrals = 0;
}


int EnergyPrintStatistics(char* s, int printHeader, tStatEnergyItem *statItem)
{
	double mean = (double)statItem->sumPulseLength / (double)statItem->countPulses;
	double std = sqrt( ((double)statItem->sumPulseLengthPow2/(double)statItem->countPulses) - mean*mean );


	int charNum = 0;
	charNum += sprintf(s + charNum,     "DELKA PULZU          min:  %4d     mean:  %4.0f     std:  %4.1f\n", statItem->minPulseLength, mean, std);
	charNum += sprintf(s + charNum,     "PRILIS KRATKY PULZ   na zacatku: %8d   na konci: %8d\n",statItem->numToShortPulsesAtBegining, statItem->numToShortPulsesAtEnding);
	charNum += sprintf(s + charNum,     "VYPOCET ENERGIE      prilis vysoka hodnota integralu: %8d  \n", statItem->numSaturatedIntegrals);

	return charNum;
}


float EnergyFromIntegral2(float *impuls, int delka, int poziceMax, tStatEnergy *stats, int *prilisKratkyPulzProIntegraci)
{
	//stats - pulse length
	stats->actual.minPulseLength = (stats->actual.minPulseLength < delka) ? stats->actual.minPulseLength : delka;
	stats->total.minPulseLength = (stats->total.minPulseLength < delka) ? stats->total.minPulseLength : delka;
	stats->actual.sumPulseLength += (unsigned long long)delka;
	stats->total.sumPulseLength += (unsigned long long)delka;
	stats->actual.sumPulseLengthPow2 += (unsigned long long)(delka*delka);
	stats->total.sumPulseLengthPow2 += (unsigned long long)(delka*delka);
	stats->actual.countPulses++;
	stats->total.countPulses++;

	*prilisKratkyPulzProIntegraci = 0;


	//start integrating
	int start = poziceMax - 35;
	if (start < 0)
	{
		start = 0;
		*prilisKratkyPulzProIntegraci = 1;

		//statistics - to short pulse at beginning
		stats->actual.numToShortPulsesAtBegining++;
		stats->total.numToShortPulsesAtBegining++;
	}


	//end integrating
	int stop = poziceMax + 350;
	if (stop > delka-1)
	{
		stop = delka - 1;
		*prilisKratkyPulzProIntegraci = 1;

		//statistics - to long pulse at ending
		stats->actual.numToShortPulsesAtEnding++;
		stats->total.numToShortPulsesAtEnding++;
	}


	//energy from integral
	float energy = 0;
	for (int a = start; a < stop; a++)
	{
		energy += impuls[a];
	}
	energy /= 20.0;		//the value of integral is bigger than amplitude of impulse. This constant makes the spectrum very simmilar to amplitude spectrum.


	return energy;
}



void SaturateEnergy(float *energyAmplitude, float *energyIntegral, tStatEnergy *stats)
{
	int saturatedIntegral = 0;
	int saturatedAmplitude = 0;

	//bad computed integral
	if (*energyIntegral > 3950)
	{
		*energyIntegral = 4000;
		saturatedIntegral = 1;
	}

	//saturated value on ADC
	if (*energyAmplitude > 3500)
	{
		*energyAmplitude = 4050;
		*energyIntegral = 4050;
		saturatedAmplitude = 1;
	}

	//statistics
	if (saturatedIntegral && !saturatedAmplitude)
	{
		stats->actual.numSaturatedIntegrals++;
		stats->total.numSaturatedIntegrals++;
	}
}
