/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ClassificatorScript - spectrometric data processing
 *
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>
 *
 *  Date      :
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#ifndef __STATS_S
#define __STATS_S

#include "energy.h"
#include "params.h"
#include "dynamZero.h"
#include "branchFusing.h"


typedef struct sStatTimeLossItem
{
    double minimum_sec;
    double maximum_sec;
    double sum_sec;
    double sumPow2_sec;
    unsigned long long numLoss;
    unsigned long long numLossImpulses;
} tStatTimeLossItem;

typedef struct sStatTimeItem
{
	tStatTimeLossItem shortLoss;	//random loss of one or more packets
	tStatTimeLossItem mediumLoss;	//loss caused by filling the FIFO in FPGA
	tStatTimeLossItem longLoss;		//loss caused by filling RAM in computer

	double timeFirstImpulse_sec;
	double timeLastImpulse_sec;
	unsigned long long numberAllImpulses;
	unsigned long long numberSuperposedImpulses;
	unsigned long long numberNotCompletedImpulses;

} tStatTimeItem;


typedef struct sStatTime
{
	tStatTimeItem actual;
	tStatTimeItem total;

} tStatTime;


void TimeClearStatistics(tStatTime *stats, int onlyActual);
int TimePrintStatistics(char* s, tStatTimeItem *statItem);







void PrintStatistics(ParamsStruct *params);

int ZastavZpracovani(ParamsStruct *params);

void WriteAllStatsToString(unsigned long long casImpulzu_ns, ParamsStruct *params, tStatDynamZero *DynamZeroStatCH1, tStatDynamZero *DynamZeroStatCH2, tBranchFusingStat *branchFusingStats, tStatEnergy *energyStats, tStatTime *timeStats);

void PrintTotalStats(ParamsStruct *params, tStatDynamZero *DynamZeroStatCH1, tStatDynamZero *DynamZeroStatCH2, tBranchFusingStat *branchFusingStats, tStatEnergy *energyStats, tStatTime *timeStats);

void TimeStatsOfSavedImpulses(tStatTime *stats, unsigned long long casImpulzu_ns, int pulzKompletni, unsigned long long vypadek_ns, int vypadek_pulzy, int superpozice);
void SetTimeOfFirstReceivedPulse(unsigned long long time);

int OpenStatisticFile(char* filePrefix);
FILE* GetStatisticFile();
void CloseStatisticFile();

#endif //__STATS_S
